"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getGraphQLParams = void 0;
const graphql_1 = require("graphql");
const httpError = (statusCode, message, errProps = {}) => {
    let err = new Error(message);
    err = { ...err, ...errProps };
    return err;
};
async function graphqlWXServer(options) {
    var _a, _b, _c, _d, _e, _f, _g, _h;
    // Higher scoped variables are referred to at various stages in the asynchronous state machine below.
    let params;
    let formatErrorFn = graphql_1.formatError;
    let pretty = false;
    let result;
    try {
        // Parse the Request to get GraphQL request parameters.
        params = await getGraphQLParams(options.wxParams);
        const optionsData = options;
        const schema = optionsData.schema;
        const rootValue = optionsData.rootValue;
        const validationRules = (_a = optionsData.validationRules) !== null && _a !== void 0 ? _a : [];
        const fieldResolver = optionsData.fieldResolver;
        const typeResolver = optionsData.typeResolver;
        // const extensionsFn = optionsData.extensions;
        const context = optionsData.wxContext;
        const parseFn = (_b = optionsData.customParseFn) !== null && _b !== void 0 ? _b : graphql_1.parse;
        const executeFn = (_c = optionsData.customExecuteFn) !== null && _c !== void 0 ? _c : graphql_1.execute;
        const validateFn = (_d = optionsData.customValidateFn) !== null && _d !== void 0 ? _d : graphql_1.validate;
        pretty = (_e = optionsData.pretty) !== null && _e !== void 0 ? _e : false;
        formatErrorFn = (_g = (_f = optionsData.customFormatErrorFn) !== null && _f !== void 0 ? _f : optionsData.formatError) !== null && _g !== void 0 ? _g : formatErrorFn;
        // Assert that schema is required.
        if (schema == null) {
            throw httpError(500, 'GraphQL middleware options must contain a schema.');
        }
        // Get GraphQL params from the wx-cloud-server event
        const { query, variables, operationName } = params;
        // If there is no query, but GraphiQL will be displayed, do not produce
        // a result, otherwise return a 400: Bad Request.
        if (query == null) {
            throw httpError(400, 'Must provide query string.');
        }
        // Validate Schema
        const schemaValidationErrors = graphql_1.validateSchema(schema);
        if (schemaValidationErrors.length > 0) {
            // Return 500: Internal Server Error if invalid schema.
            throw httpError(500, 'GraphQL schema validation error.', {
                graphqlErrors: schemaValidationErrors,
            });
        }
        // Parse source to AST, reporting any syntax error.
        let documentAST;
        try {
            documentAST = parseFn(new graphql_1.Source(query, 'GraphQL request'));
        }
        catch (syntaxError) {
            // Return 400: Bad Request if any syntax errors errors exist.
            throw httpError(400, 'GraphQL syntax error.', {
                graphqlErrors: [syntaxError],
            });
        }
        // Validate AST, reporting any errors.
        const validationErrors = validateFn(schema, documentAST, [
            ...graphql_1.specifiedRules,
            ...validationRules,
        ]);
        if (validationErrors.length > 0) {
            // Return 400: Bad Request if any validation errors exist.
            throw httpError(400, 'GraphQL validation error.', {
                graphqlErrors: validationErrors,
            });
        }
        // Perform the execution, reporting any errors creating the context.
        try {
            result = await executeFn({
                schema,
                document: documentAST,
                rootValue,
                contextValue: context,
                variableValues: variables,
                operationName,
                fieldResolver,
                typeResolver,
            });
        }
        catch (contextError) {
            // Return 400: Bad Request if any execution context errors exist.
            throw httpError(400, 'GraphQL execution context error.', {
                graphqlErrors: [contextError],
            });
        }
    }
    catch (error) {
        result = { data: undefined, errors: (_h = error.graphqlErrors) !== null && _h !== void 0 ? _h : [error] };
    }
    return result;
}
exports.default = graphqlWXServer;
/**
 *  Provided the event from wx-cloud-server main func param
 *  Promise the GraphQL request parameters.
 */
async function getGraphQLParams(param) {
    // GraphQL Query string.
    let query = param.query;
    if (typeof query !== 'string') {
        query = null;
    }
    // Parse the variables if needed.
    let variables = param.variables;
    if (typeof variables === 'string') {
        try {
            variables = JSON.parse(variables);
        }
        catch (error) {
            throw httpError(400, 'Variables are invalid JSON.');
        }
    }
    else if (typeof variables !== 'object') {
        variables = null;
    }
    // Name of GraphQL operation to execute.
    let operationName = param.operationName;
    if (typeof operationName !== 'string') {
        operationName = null;
    }
    const raw = param.raw;
    return { query, variables, operationName, raw };
}
exports.getGraphQLParams = getGraphQLParams;
